<?php

namespace FiloBlu\Refilo\Controller\Adminhtml\Exporter;

use Exception;
use FiloBlu\Refilo\Model\Indexer\Core\AbstractMongoIndexer;
use Magento\Framework\App\Action\HttpGetActionInterface as HttpGetActionInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Message\ManagerInterface;

class Countries implements HttpGetActionInterface
{
    /**
     * @var AbstractMongoIndexer
     */
    private $abstractMongoIndexer;
    /**
     * @var
     */
    private $messageManager;
    /**
     * @var ResultFactory
     */
    private $resultFactory;

    public function __construct(
        AbstractMongoIndexer $abstractMongoIndexer,
        ResultFactory $resultFactory,
        ManagerInterface $messageManager
    ) {
        $this->abstractMongoIndexer = $abstractMongoIndexer;
        $this->resultFactory = $resultFactory;
        $this->messageManager = $messageManager;
    }

    public function execute()
    {
        try {
            $this->abstractMongoIndexer->executeFull();
        } catch (LocalizedException $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        } catch (Exception $e) {
            $this->messageManager->addExceptionMessage($e, __('An error occurred while syncing data with MongoDB.'));
        }

        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $this->messageManager->addSuccessMessage(__('Countries data successfully synced with mongodb'));
        return $resultRedirect->setPath('indexer/indexer/list/');
    }
}
