<?php

namespace FiloBlu\Refilo\Model\Indexer;

use FiloBlu\Refilo\Helper\Data as HelperData;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Indexer\Processor\Catalog as ProcessorCatalog;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Mview\ActionInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * @property ResourceConnection _resource
 * @property StoreManagerInterface _storeManager
 */
class Catalog implements \Magento\Framework\Indexer\ActionInterface, ActionInterface
{
    /**
     * @var ProcessorCatalog
     */
    private $processorCatalog;

    /**
     * @var integer
     */
    private $batchSize;
    /**
     * @var integer
     */
    private $batchSequence;
    /**
     * @var Mongo
     */
    private $connection;
    /**
     * @var FiloBlu\Refilo\Helper\Data
     */
    private $helperData;

    /**
     * Catalog constructor.
     * @param StoreManagerInterface $storeManager
     * @param ProcessorCatalog $processorCatalog
     * @param Mongo $connection
     * @param HelperData $helperData
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        ProcessorCatalog $processorCatalog,
        Mongo $connection,
        HelperData $helperData
    ) {
        $this->connection = $connection;
        $this->helperData = $helperData;
        $this->_storeManager = $storeManager;
        $this->processorCatalog = $processorCatalog;
        $this->batchSize = ($batch_size = (int)$this->helperData->getConfigValue("sync/data/batch_size")) ? $batch_size : 100;

        $this->batchSequence = 1;
    }

    /*
     * Used by mview, allows process indexer in the "Update on schedule" mode
     */
    public function execute($ids)
    {
        $this->executeGeneric($ids);
    }

    /**
     * @param bool $ids
     * @throws \MongoDB\Driver\Exception\Exception
     */
    protected function executeGeneric($ids = false)
    {
        $storeDataList = $this->_storeManager->getStores();
        echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640

        foreach ($storeDataList as $key => $store) {
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $starttime = $mtime;
            printf("Prepare for store %s", $store->getCode() . "\n\r");
            while ($productItems = $this->processorCatalog->getProducts($store->getId(), $this->batchSize, $this->batchSequence, $ids)) {
                echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
                $mongo_collection = "products_" . $store->getCode();
                $this->connection->update("_id", $productItems, $mongo_collection, true);
                $this->batchSequence++;
            }
            $this->processorCatalog->setTotalPages(0);
            $this->batchSequence = 1;
            $this->connection->ensureIndex($store->getCode());
            $mtime = microtime();
            $mtime = explode(" ", $mtime);
            $mtime = $mtime[1] + $mtime[0];
            $endtime = $mtime;
            $totaltime = ($endtime - $starttime);
            echo memory_get_usage() / 1024 / 1024 . "\n"; // 36640
            echo "Executed in " . $totaltime . " seconds";
        }
    }

    public function executeFull()
    {
        $this->executeGeneric();
    }

    /*
     * Works with a set of entity changed (may be massaction)
     */

    public function executeList(array $ids)
    {
        //Works with a set of placed orders (mass actions and so on)
        $this->executeGeneric($ids);
    }

    /*
     * Works in runtime for a single entity using plugins
     */
    public function executeRow($id)
    {
        //Works in runtime for a single order using plugins
        $this->executeGeneric([$id]);
    }

    /**
     * Returns table name
     *
     * @param string|array $name
     * @return string
     */
    public function getTable($name)
    {
        return $this->_resource->getTableName($name);
    }
}
